﻿# ---- to do
# resize the chairs
# resize the dishes
# returning dishes

init python:
    # preferences.gl_framerate = 60
    def aabb_collision(box_1, box_2):
        if box_1.x < box_2.x + box_2.width:
            return False
        if box_1.x + box_1.width > box_2.x:
            return False
        if box_1.y < box_2.y + box_2.height:
            return False
        if box_1.y + box_1.height > box_2.y:
            return False
        return True

    def load_map(tilemap):
        map = []
        f = open(renpy.loader.transfn(tilemap),"r")
        for line in f:
            line = line.strip()
            divided = line.split(",")
            listed = [int(x) for x in divided]
            map.append(listed)
        f.close()
        return map
    def load_tiles(img, size, y = 8, x = 8):
        tiles = []
        for i in range(x):
            for ii in range(y):
                tiles.append(Crop((ii*size,i*size,size,size), img))
        return tiles

    def construct_map(map, size):
        result = []
        for y,j in enumerate(map):
            for x,i in enumerate(j):
                result.append(tile(i, x*size, y*size, size))
        return result

    def make_collision_map(map, floors):
        result = []
        for y in map:
            row = []
            for x in y:
                if x in floors:
                    row.append(0)
                else:
                    row.append(1)
            result.append(row)
        return result

    def find_tables(map, tables, chairs):
        table_list = []
        for y,j in enumerate(map):
            for x,i in enumerate(j):
                if i in tables:
                    table = {
                        "x": x,
                        "y": y,
                        "left_chair": None,
                        "right_chair": None,
                        "food": None,
                    }
                    if map[y][x-1] in chairs:
                        table["left_chair"] = topdown_client(x-1, y, -1)
                    if map[y][x+1] in chairs:
                        table["right_chair"] = topdown_client(x+1, y, 1)

                    table_list.append(table)
        return table_list


    class topdown_client:
        def __init__(self, x, y, xzoom):
            self.x = x
            self.y = y
            self.xzoom = xzoom

            self.food_request = None

            self.image = None
            self.wait = 0
            self.leaving = 3
        def tick(self):
            self.wait -=1

    class animation_class:
        def __init__(self, frames):
            self.frames = frames
            self.current_delta = 0
            self.current_index = 0
        def get_frame(self, delta):
            self.current_delta += delta
            current_frame = self.frames[self.current_index]
            if self.current_delta > current_frame[1]:
                self.current_delta = 0
                self.current_index += 1
                if self.current_index > len(self.frames)-1:
                    self.current_index = 0
            return self.frames[self.current_index][0]

    class animation_handler:
        def __init__(self, animations):
            self.animations = animations
            self.current_animation = None

        def get_frame(self, delta, animation):
            if type(self.animations[animation]) is str:
                return self.animations[animation]
            if not self.current_animation == animation:
                self.current_animation = animation
                return self.animations[animation].get_frame(delta)
            return self.animations[animation].get_frame(delta)
            
    class tile:
        def __init__(self, image, x, y, size):
            self.image = image
            self.x = x
            self.y = y
            self.max_x = x + size
            self.max_y = y + size

    class actor_obj(renpy.Displayable):
        def __init__(self, animations, parent, x, y):
            super(renpy.Displayable,self).__init__()
            self.animations = animations
            self.x = x
            self.y = y
            self.speed = 90
            self.facing = 0
            self.frame = 0
            self.animation = "down_idle"
            self.last_time = 0
            self.delta_time = 0
            self.parent = parent
            self.holding = None

        def move(self, move):
            self.animation = move
        def stop(self):
            if self.animation in ["up_idle", "down_idle", "left_idle", "right_idle"]:
                return
            self.animation = self.animation + "_idle"
        def render(self,width,height,st,at):
            if self.last_time == 0:
                self.last_time = time.perf_counter()
            current_time = time.perf_counter()
            delta_time =  round(current_time - self.last_time, 4)
            self.delta_time = delta_time
            if self.animation in ["up_idle", "down_idle", "left_idle", "right_idle"]:
                pass
            else:
                x = int(self.x + 16)
                y = int(self.y + 22)
                if self.animation == "up":
                    left_hand_x, left_hand_y = x - 8 , y - 10
                    right_hand_x, right_hand_y = x + 8 , y - 10
                    if self.parent.collision_map[left_hand_y//32][left_hand_x//32] or self.parent.collision_map[right_hand_y//32][right_hand_x//32]:
                        self.stop()
                    else:
                        self.y -= self.speed*delta_time
                elif self.animation == "down":
                    left_hand_x, left_hand_y = x + 8 , y + 10
                    right_hand_x, right_hand_y = x - 8 , y + 10
                    if self.parent.collision_map[left_hand_y//32][left_hand_x//32] or self.parent.collision_map[right_hand_y//32][right_hand_x//32]:
                        self.stop()
                    else:
                        self.y += self.speed*delta_time
                elif self.animation == "right":
                    left_hand_x, left_hand_y = x + 10 , y + 8
                    right_hand_x, right_hand_y = x + 10 , y - 8
                    if self.parent.collision_map[left_hand_y//32][left_hand_x//32] or self.parent.collision_map[right_hand_y//32][right_hand_x//32]:
                        self.stop()
                    else:
                        self.x += self.speed*delta_time
                elif self.animation == "left":
                    left_hand_x, left_hand_y = x - 10 , y - 8
                    right_hand_x, right_hand_y = x - 10 , y + 8
                    if self.parent.collision_map[left_hand_y//32][left_hand_x//32] or self.parent.collision_map[right_hand_y//32][right_hand_x//32]:
                        self.stop()
                    else:
                        self.x -= self.speed*delta_time
            

            image = self.animations.get_frame(delta_time * 1000, self.animation)
            if self.holding:
                img = Composite(
                    (32,32),
                    (0,0), image,
                    (0,10), self.holding
                )
            else:
                img = image
            self.last_time = current_time
            t = Transform(img, xanchor=0.5, yanchor=0.5)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (self.x, self.y))

            renpy.redraw(self, 0)
            return rv

    class topdown_handler:
        def __init__(self, map, tileset, size, clients, all_food, counter, duration, start):
            self.stat = "playing"
            self.player = actor_obj(main_animation_frames, self,start[0]*32, start[1]*32)
            self.current_move = None

            self.map = load_map(map)
            self.collision_map = make_collision_map(self.map, [18])
            self.tileset = load_tiles(tileset, size, 8, 4)
            self.size = size
            self.tables = find_tables(self.map, [21], [20,22])
            
            self.clients = clients
            self.all_food = all_food
            self.counter = counter

            self.requested_food = []
            self.happy = 0
            self.angry = 0
            self.time = duration
            self.init_time = duration

        def reset(self, duration = None):
            self.requested_food = []
            self.happy = 0
            self.angry = 0
            if duration:
                self.init_time = duration
            self.time = self.init_time
            for i in self.counter:
                i[2] = None
            self.stat = "start"
        def check_for_end(self):
            if self.time > 0:
                return False
            for i in self.tables:
                if (i["left_chair"] and i["left_chair"].image) or (i["right_chair"] and i["right_chair"].image):
                    return False
            return True
        def tick(self):
            self.spawn_food()
            self.time -= 1
            if self.check_for_end():
                self.stat = "done"
                return
            for i in self.tables:
                left = i["left_chair"]
                right = i["right_chair"]
                if  left and left.image:
                    if not left.food_request:
                        food = random.choice(self.all_food)
                        left.food_request = food
                        self.requested_food.append(food)
                    elif left.food_request == i["food"]:
                        renpy.play("topdown/eat.ogg", "sfx1")
                        i["food"] = None
                        left.food_request = "eaten"
                    elif left.food_request == "eaten":
                        left.leaving -= 1
                        if left.leaving == 2:
                            floating_text("topdown_hearts", i["x"]-1, i["y"])
                            renpy.play("topdown/level_up.ogg", "sfx2")
                        elif left.leaving < 1:
                            left.image = None
                            left.food_request = None
                            self.happy += 1
                    else:
                        left.wait -= 1
                        if left.wait == 1:
                            floating_text("topdown_angry", i["x"]-1, i["y"])
                            renpy.play("topdown/wrong.ogg", "sfx1")
                        if left.wait < 0:
                            left.image = None
                            left.food_request = None
                            self.angry += 1


                if right and right.image:
                    if not right.food_request:
                        food = random.choice(self.all_food)
                        right.food_request = food
                        self.requested_food.append(food)
                    elif right.food_request == i["food"]:
                        renpy.play("topdown/eat.ogg", "sfx1")
                        i["food"] = None
                        right.food_request = "eaten"
                    elif right.food_request == "eaten":
                        right.leaving -= 1
                        if right.leaving == 2:
                            floating_text("topdown_hearts", i["x"]+1, i["y"])
                            renpy.play("topdown/level_up.ogg", "sfx2")
                        elif right.leaving < 1:
                            right.image = None
                            right.food_request = None
                            self.happy += 1
                    else:
                        right.wait -= 1
                        if right.wait == 1:
                            floating_text("topdown_angry", i["x"]+1, i["y"])
                            renpy.play("topdown/wrong.ogg", "sfx1")
                        if right.wait < 0:
                            right.image = None
                            right.food_request = None
                            self.angry += 1

            self.spawn_client()


        def take(self):
            x = int((self.player.x+16)//32)
            y = int(self.player.y//32)
            for i in self.counter:
                xdist = i[0] - x
                ydist = i[1] - y
                if abs(xdist)+abs(ydist) == 0:
                    self.player.holding, i[2] = i[2], self.player.holding
                    renpy.play("topdown/move.ogg", "sound")
                    return
            for i in self.tables:
                xdist = i["x"] - x
                ydist = i["y"] - y
                if abs(xdist)+abs(ydist) < 2:
                    self.player.holding, i["food"] = i["food"], self.player.holding
                    renpy.play("topdown/move.ogg", "sound")
                    return

        def spawn_food(self):
            if not len(self.requested_food):
                return
            food = self.requested_food.pop(0)
            for i in self.counter:
                if i[2] is None:
                    i[2] = food
                    return
            self.requested_food.append(food)
        def spawn_client(self):
            if self.time < 1:
                return
            table = random.choice(self.tables)
            client = random.choice(self.clients)
            wait = random.randint(15,25)
            if random.choice([True, False]):
                if table["left_chair"] and not table["left_chair"].image:
                    table["left_chair"].image = client
                    table["left_chair"].wait = wait
                    table["left_chair"].leaving = 3
            else:
                if table["right_chair"] and not table["right_chair"].image:
                    table["right_chair"].image = client
                    table["right_chair"].wait = wait
                    table["right_chair"].leaving = 3
        def move(self, move):
            if self.current_move == move:
                self.current_move = None
                self.player.stop()
                return
            self.current_move = move
            self.player.move(move)

screen topdown_screen(g):
    style_prefix "topdown"
    fixed xysize 640,352:
        at zoom(3, True)
        vbox spacing 0 align (0.5, 0.5):
            for yp,y in enumerate(g.map):
                hbox spacing 0:
                    for xp,x in enumerate(y):
                        frame xysize(g.size,g.size):
                            if x in [-1]:
                                background None
                            else:
                                background g.tileset[x]
                            # text str(g.collision_map[yp][xp]) size 10 align .5,.5
        for i in g.tables:
            if i["left_chair"]:
                fixed fit_first True:
                    pos i["left_chair"].x*32, i["left_chair"].y*32 yoffset -4
                    add i["left_chair"].image at flip(i["left_chair"].xzoom)
                    if i["left_chair"].food_request:
                        add i["left_chair"].food_request at zoom(.5) offset -4,-4
                    if i["left_chair"].image:
                        bar value i["left_chair"].wait range 30 xysize 32,1

            if i["right_chair"]:
                fixed fit_first True:
                    pos i["right_chair"].x*32, i["right_chair"].y*32 yoffset -4
                    add i["right_chair"].image  at flip(i["right_chair"].xzoom)
                    if i["right_chair"].food_request:
                        add i["right_chair"].food_request at zoom(.5) xalign 1.0 offset 4,-4
                    if i["right_chair"].image:
                        bar value i["right_chair"].wait range 30 xysize 32,1
            add i["food"] pos i["x"]*32, i["y"]*32 yoffset -10
        for i in g.counter:
            add i[2] pos i[0]*32,i[1]*32 yoffset -12
        add g.player

    vbox:
        hbox spacing 10:
            add "topdown_hearts" at zoom(2, True)
            text str(g.happy)
        hbox spacing 10:
            add "topdown_angry" at zoom(2, True)
            text str(g.angry)
    bar value g.time range g.init_time xysize 1400,20 align .5,.0 yoffset 40
    fixed xysize 100,100 align 0.0,1.0 offset 150,-150:
        button xysize 100,100:
            yoffset -100 background "#fff5"
            text "w" align .5,.5
            action Function(g.move, "up") keysym ["K_UP", "w"]
        button xysize 100,100:
            xoffset -100 background "#fff5"
            text "a" align .5,.5
            action Function(g.move, "left") keysym ["K_LEFT", "a"]
        button xysize 100,100:
            yoffset 100 background "#fff5"
            text "s" align .5,.5
            action Function(g.move, "down") keysym ["K_DOWN", "s"]
        button xysize 100,100:
            xoffset 100 background "#fff5"
            text "d" align .5,.5
            action Function(g.move, "right") keysym ["K_RIGHT", "d"]

    button xysize 100,100:
        align 1.0,1.0 offset -150,-150
        background "#fff5"
        text "e" align .5,.5
        action Function(g.take) keysym ["e"]
    if g.stat == "playing":
        timer 2 repeat True action Function(g.tick)
    elif g.stat == "done":
        frame:
            style_prefix "topdown_ui"
            vbox spacing 20:
                text "The shift is over."
                button:
                    text "Done"
                    action Return([g.happy, g.angry])                    

init: #styles
    style topdown_text:
        align (.5,.5)
    style topdown_frame:
        align (.5,.5)
    style topdown_button:
        align (.5,.5)

    style topdown_ui_text:
        align (.5,.5) color "#000"
    style topdown_ui_frame:
        align (.5,.5) background "#fffb" padding (20,20)
    style topdown_ui_button:
        align (.5,.5) background "#fffb" padding (20,20)

init: # clients
    image client_1:
        Crop((0, 32, 32, 32), "topdown/c1.png")
        pause .4
        Crop((32, 32, 32, 32), "topdown/c1.png")
        pause .4
        Crop((64, 32, 32, 32), "topdown/c1.png")
        pause .4
        repeat
    image client_2:
        Crop((0, 32, 32, 32), "topdown/c2.png")
        pause .4
        Crop((32, 32, 32, 32), "topdown/c2.png")
        pause .4
        Crop((64, 32, 32, 32), "topdown/c2.png")
        pause .4
        repeat
    image client_3:
        Crop((0, 32, 32, 32), "topdown/c3.png")
        pause .4
        Crop((32, 32, 32, 32), "topdown/c3.png")
        pause .4
        Crop((64, 32, 32, 32), "topdown/c3.png")
        pause .4
        repeat
    image client_4:
        Crop((0, 32, 32, 32), "topdown/c4.png")
        pause .4
        Crop((32, 32, 32, 32), "topdown/c4.png")
        pause .4
        Crop((64, 32, 32, 32), "topdown/c4.png")
        pause .4
        repeat
    image client_5:
        Crop((0, 32, 32, 32), "topdown/c5.png")
        pause .4
        Crop((32, 32, 32, 32), "topdown/c5.png")
        pause .4
        Crop((64, 32, 32, 32), "topdown/c5.png")
        pause .4
        repeat
    image client_6:
        Crop((0, 32, 32, 32), "topdown/c6.png")
        pause .4
        Crop((32, 32, 32, 32), "topdown/c6.png")
        pause .4
        Crop((64, 32, 32, 32), "topdown/c6.png")
        pause .4
        repeat
    default all_clients = ["client_1", "client_2", "client_3", "client_4", "client_5", "client_6"]
init: # main character
    image main_down_1:
        Crop((0, 0, 32, 32), "topdown/main.png")
    image main_down_2:
        Crop((32, 0, 32, 32), "topdown/main.png")
    image main_down_3:
        Crop((64, 0, 32, 32), "topdown/main.png")

    image main_left_1:
        Crop((0, 32, 32, 32), "topdown/main.png")
    image main_left_2:
        Crop((32, 32, 32, 32), "topdown/main.png")
    image main_left_3:
        Crop((64, 32, 32, 32), "topdown/main.png")

    image main_up_1:
        Crop((0, 64, 32, 32), "topdown/main.png")
    image main_up_2:
        Crop((32, 64, 32, 32), "topdown/main.png")
    image main_up_3:
        Crop((64, 64, 32, 32), "topdown/main.png")

    image main_right_1:
        Crop((0, 96, 32, 32), "topdown/main.png")
    image main_right_2:
        Crop((32, 96, 32, 32), "topdown/main.png")
    image main_right_3:
        Crop((64, 96, 32, 32), "topdown/main.png")

    default main_animation_up = animation_class(
            [
                ["main_up_1", 150],
                ["main_up_2", 150],
                ["main_up_3", 150],
                ["main_up_2", 150],
            ]
        )
    default main_animation_left = animation_class(
            [
                ["main_left_1", 150],
                ["main_left_2", 150],
                ["main_left_3", 150],
                ["main_left_2", 150],
            ]
        )
    default main_animation_down = animation_class(
            [
                ["main_down_1", 150],
                ["main_down_2", 150],
                ["main_down_3", 150],
                ["main_down_2", 150],
            ]
        )
    default main_animation_right = animation_class(
            [
                ["main_right_1", 150],
                ["main_right_2", 150],
                ["main_right_3", 150],
                ["main_right_2", 150],
            ]
        )
    
    default main_animation_frames = animation_handler(
            {
                "up": main_animation_up,
                "left": main_animation_left,
                "down": main_animation_down,
                "right": main_animation_right,

                "up_idle": "main_up_2",
                "left_idle": "main_left_2",
                "down_idle": "main_down_2",
                "right_idle": "main_right_2",
            }
        )

init: # food
    image topdown_angry = Crop((224, 32, 32, 32), "topdown/atlas.png")
    image topdown_hearts = Crop((224, 0, 32, 32), "topdown/atlas.png")
    image eaten = Crop((0, 128, 32, 32), "topdown/atlas.png")
    
    image topdown_burger = Crop((32, 128, 32, 32), "topdown/atlas.png")
    image topdown_eggs = Crop((64, 128, 32, 32), "topdown/atlas.png")
    image topdown_hotdog = Crop((96, 128, 32, 32), "topdown/atlas.png")
    image topdown_pankakes = Crop((128, 128, 32, 32), "topdown/atlas.png")
    image topdown_doughnuts = Crop((160, 128, 32, 32), "topdown/atlas.png")
    image topdown_taco = Crop((192, 128, 32, 32), "topdown/atlas.png")
    image topdown_pizza = Crop((224, 128, 32, 32), "topdown/atlas.png")

    image topdown_ham = Crop((0, 160, 32, 32), "topdown/atlas.png")
    image topdown_cake = Crop((32, 160, 32, 32), "topdown/atlas.png")
    image topdown_cookies = Crop((64, 160, 32, 32), "topdown/atlas.png")
    image topdown_dango = Crop((96, 160, 32, 32), "topdown/atlas.png")
    image topdown_lobster = Crop((128, 160, 32, 32), "topdown/atlas.png")
    image topdown_sushi = Crop((160, 160, 32, 32), "topdown/atlas.png")
    image topdown_rice = Crop((192, 160, 32, 32), "topdown/atlas.png")
    default all_food = [
        "topdown_burger",
        "topdown_eggs",
        "topdown_hotdog",
        "topdown_pankakes",
        "topdown_doughnuts",
        "topdown_taco",
        "topdown_pizza",
        "topdown_ham",
        "topdown_cake",
        "topdown_cookies",
        "topdown_dango",
        "topdown_lobster",
        "topdown_sushi",
        # "topdown_rice",
        ]

default location_1 = topdown_handler(
    map = "topdown/map_1.csv",
    tileset = "topdown/atlas.png",
    size = 32,
    clients = all_clients,
    all_food = all_food,
    counter = [[5,3,None], [6,3,None], [7,3,None], [8,3,None], [9,3,None]],
    duration = 40,
    start = (7,6),
    )
default location_2 = topdown_handler(
    map = "topdown/map_2.csv",
    tileset = "topdown/atlas.png",
    size = 32,
    clients = all_clients,
    all_food = all_food,
    counter = [[3,4,None], [3,5,None], [3,6,None], [3,7,None], [16,5,None], [16,6,None]],
    duration = 40,
    start = (6,5),
    )
define Madelyn = Character("Madelyn")

label topdown_example:
    scene bg room with dissolve
    show madelyn normal with dissolve
    Madelyn "Ready to start working as a waiter?"
    Madelyn "All you have to do is deliver the fod to the right client."
    Madelyn "Good luck."
    hide madelyn with dissolve
    call screen topdown_screen(location_1)
    show madelyn normal with dissolve
    Madelyn "Served [_return[0]] and have [_return[1]] angry clients."
    Madelyn "Corporate wants you in the other branch."
    Madelyn "It's bigger and there's a bug with vertical counters that needs fixing."
    Madelyn "But there's no time to chitchat. Get going."
    hide madelyn with dissolve
    call screen topdown_screen(location_2)
    "Served [_return[0]] and have [_return[1]] angry clients."



